/*
 * Zero driver
 * Copyright (c) 2010 Michal Nazarewicz (mina86@mina86.com)
 */

#include <linux/module.h>
#include <linux/miscdevice.h>
#include <linux/fs.h>
#include <linux/uaccess.h>

MODULE_DESCRIPTION("Example driver doing what /dev/zero does");
MODULE_AUTHOR("Michal Nazarewicz");
MODULE_LICENSE("GPL");


static const char zeros[PAGE_SIZE];

static ssize_t zero_read(struct file *file, char __user *buf,
						 size_t len, loff_t *offp)
{
	ssize_t ret = min(len, sizeof zeros);
	printk(KERN_INFO "zero: read(%zu (%zd))\n", len, ret);

	if (likely(ret))
		if (unlikely(__copy_to_user(buf, zeros, ret)))
			ret = -EFAULT;

	return ret;
}

static ssize_t zero_write(struct file *file, const char __user *buf,
						  size_t len, loff_t *offp)
{
	printk(KERN_INFO "zero: write(%zu)\n", len);

	return len;
}


static const struct file_operations zero_fops = {
	.read    = zero_read,
	.write   = zero_write,
};


static struct miscdevice zero_dev = {
	.minor = MISC_DYNAMIC_MINOR,
	.name  = "myzero",
	.fops  = &zero_fops,
};


static int __init zero_init(void)
{
	int ret = misc_register(&zero_dev);
	printk(KERN_INFO "zero: loading, misc_register returned %d\n", ret);
	return ret;
}
module_init(zero_init)

static void __exit zero_exit(void)
{
	int ret = misc_deregister(&zero_dev);
	printk(KERN_INFO "zero: unloading, misc_deregister returned %d\n", ret);
}
module_exit(zero_exit)
